local frame = CreateFrame("Frame");
--frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED"); -- lets try to run less often than the old version!
frame:RegisterEvent("PLAYER_REGEN_DISABLED"); -- enter combat
frame:RegisterEvent("PLAYER_ENTERING_WORLD");
frame:RegisterEvent("ZONE_CHANGED_NEW_AREA"); 
frame:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
frame:RegisterEvent("PLAYER_TALENT_UPDATE");

--local lastWarned = GetTime();
local lastSound = 0;
if debugmode == nil then 
	debugmode = false;
end

-- we could also add a color for each command and it's description
local cmdListFormat = '|cffffcc00%s|r';
local soundListFormat = '|cff32cd32%s|r';
local valueDisplayFormat = '|cff00ff00%s|r';
local cmdList = {
  [1] =  { name = 'status', descr = 'display the current settings' },
  [2] =  { name = 'test', descr = 'see what a warning looks like' },
  [3] =  { name = 'et', descr = '"expire timer", set the expire time, in minutes, left on poison, before a warning occurs' },
  [4] =  { name = 'timer', descr = 'set the interval, in minutes, at which you are warned you have no poison' },
  [5] =  { name = 'on', descr = 'enable warning' },
  [6] =  { name = 'off', descr = 'disable warning' },
  [7] =  { name = 'wson', descr = 'enable warning sound' },
  [8] =  { name = 'wsoff', descr = 'disable warning sound ( only warning text will popup )' },
  [9] =  { name = 'wsset', descr = 'set a warning sound from a list of sounds' },
  [10] = { name = 'wsli', descr = 'listen to a warning sound from a list of sounds' },
  [11] = { name = 'toggle', descr = 'toggles the addon on and off'},
  [12] = { name = 'tu', descr = 'toggles warning for utility poison (default is "on")'},
  [13] = { name = 'slice', descr = 'seconds to elapse before slice and dice warning is displayed (0 off)' },
  [14] = { name = 'ssset', descr = 'set a slice and dice sound' },
  [15] = { name = 'debug', descr = 'troubleshooting messages'},
};
local warnSoundList = {
  [1] =  { name = "Raid Warn",   path = "Sound\\interface\\RaidWarning.ogg" },
  [2] =  { name = "Moo",         path = "Sound\\Spells\\PolyMorphCow.ogg" },
  [3] =  { name = "Moo2",        path = "Sound\\Creature\\Cow\\CowDeath.ogg" },
  [4] =  { name = "Ta-Da",       path = "Sound\\Spells\\PVPFlagTaken.ogg" },
  [5] =  { name = "Tadpole",     path = "Sound\\Creature\\BabyMurloc\\BabyMurlocA.ogg" },
  [6] =  { name = "Yahh",        path = "Sound\\Character\\Human\\MaleNPC\\HumanMalePirateHeavy\\HumanMalePirateHeavyAggro.ogg" },
  [7] =  { name = "Murmur",      path = "Sound\\Creature\\Murmur\\MurmurSpawn.ogg" },
  [8] =  { name = "Turkey",      path = "Sound\\Creature\\turkey\\TurkeyAttackA.ogg" },
  [9] =  { name = "Klunk",       path = "Sound\\Doodad\\ArcaneCrystalOpen.ogg" },
  [10] = { name = "Gong",        path = "Sound\\Doodad\\G_GongTroll01.ogg" },
  [11] = { name = "Gong2",       path = "Sound\\Doodad\\BellTollHorde.ogg" },
  [12] = { name = "Drum",        path = "Sound\\Doodad\\BellTollTribal.ogg" },
  [13] = { name = "Ding Ding",   path = "Sound\\Doodad\\BoatDockedWarning.ogg" },
  [14] = { name = "Boom",        path = "Sound\\Doodad\\Cannon01_BlastA.ogg" },
  [15] = { name = "Klank",       path = "Sound\\Doodad\\DireMaulCrystalGeneratorOpen.ogg" },
  [16] = { name = "Thunk",       path = "Sound\\Item\\Weapons\\Shields\\MetalShieldBlock2.ogg" },
  [17] = { name = "Poison",      path = "Sound\\Creature\\HydrossTheUnstable_Corrupt\\COIL_HydroCorr_SwtchCorrpt01.ogg" },
};

local playerClass, englishClass = UnitClass("player");

-- assuming savedvariables have been loaded :/
if cprexp == nil then
    cprexp = 5;
end

if utilityWarn == nil then
	utilityWarn = "on";
end

if cprTimer == nil then
    cprTimer = 1;
end

if cprTimerS == nil then
    cprTimerS = cprTimer * 60;
end

if warnSound == nil then
	warnSound = 17;
end

if warnSoundState == nil then
	warnSoundState = "on";
end

if ( englishClass == "ROGUE" and cprstatus == nil ) then
    cprstatus = "on";
else
    cprstatus = "off";
end

if slicesound == nil then
	slicesound = 6;
end

if slicewarn == nil then
	slicewarn = 5;
end
local cansnd,canpoison;

local ticker = nil;
local function StartTimer()
	if (ticker) then
		ticker:Cancel();
	end
	if (canpoison) then
		ticker = C_Timer.NewTicker(cprTimerS, CheckPoisons, nil);
	end
end

local function handler(msg)
    local command, cprrest = msg:match("^(%S*)%s*(.-)$");
	if (command == "debug") then
		debugmode = not debugmode;
		print("debug is " .. (debugmode and "true" or "false"));
    elseif command == "et" then
		if( cprrest == "" ) then
			print ("Set the number of minutes left on poison before a warning occurs. Value must be between 1 and 59. Default=5, current=" .. cprexp);
			print ("Example: /cypr et 3");
		else
	        cprrest = tonumber( cprrest );
			if ( cprrest > 0 and cprrest < 60 ) then
				cprexp = cprrest;
				print ("Expire timer has been set to " .. valueDisplay(cprexp));
			else
				print "Expire timer value must be between 1 and 59";
			end
		end
    elseif command == "timer" then
		if( cprrest == "" ) then
			print ("Set the inverval, in minutes, at which you are warned you have no poison. Value must be between 1 and 59. Default=1, current=" ..cprTimer);
			print ("Example: /cypr timer 3");
		else
	        cprrest = tonumber( cprrest );
			if ( cprrest > 0 and cprrest < 60 ) then
				cprTimer = cprrest;
				cprTimerS = cprTimer * 60;
				StartTimer();
				print ("Timer has been set to " .. valueDisplay(cprTimer));
			else
				print "Timer value must be between 1 and 59";
			end
		end
    -- test case
    elseif command == 'test' then
        SoundHandle( true , GetTime(), warnSound );
        UIErrorsFrame:AddMessage("Test Poison Running Out   ", 1.0, 1.0, 0.5, 5.0);
    -- on
	elseif command == 'tu' then
		if(utilityWarn == "on") then
			utilityWarn = "off";
			print ("Utility poison warning disabled");
		else
			utilityWarn = "on";
			print ("Utility poison warning enabled")
		end
	elseif command == 'slice' then
		slicewarn = tonumber(cprrest);
		print ("slice and dice warning",slicewarn);
    elseif command == 'on' then
        cprstatus = "on";
        print ("Warning enabled");
    -- off
    elseif command == 'off' then
        cprstatus ="off";
        print ("Warning disabled");
	-- toggle
	elseif command == 'toggle' then
		if(cprstatus == "on") then
			cprstatus ="off";
			print ("Warning disabled");
		else
			cprstatus = "on";
			print ("Warning enabled");
		end
     -- status for variables
    elseif command == 'status' then
        print ("CPR status = " .. valueDisplay(cprstatus));
        print ("Warning sound state = " .. valueDisplay(warnSoundState));
        print ("Warning sound = " .. warnSound..' - '..valueDisplay(getWSLNameByIndex(warnSound)));
        print ("Utility Poison Warning = " .. utilityWarn);
		print ("Slice and Dice Warning = " .. slicewarn);
        print ("Slice and Dice Warning sound = " .. slicesound..' - '..valueDisplay(getWSLNameByIndex(slicesound)));
    -- turn warning sound on
    elseif command == 'wson' then
		warnSoundState = "on";
		print("Warning sound has been set to " .. valueDisplay(warnSoundState));
    -- turn warning sound off
    elseif command == 'wsoff' then
		warnSoundState = "off";
		print("Warning sound has been set to " .. valueDisplay(warnSoundState));
    -- warning sound selector
    elseif command == 'wsset' then
		sndListSize = getn( warnSoundList );
		if( cprrest == "" ) then
			displayListOfSounds();
			print( 'Select a number between 1 and '..sndListSize );
		else
			cprrest = tonumber( cprrest );
			if( cprrest > 0 and cprrest <= sndListSize ) then
				warnSound = cprrest;
				print( 'Setting warning sound to: '..warnSound..' - '..string.format( soundListFormat, getWSLNameByIndex(warnSound) ) );
			else
				print( 'The value must be between 1 and '..sndListSize );
			end
		end
    elseif command == 'ssset' then
		sndListSize = getn( warnSoundList );
		if( cprrest == "" ) then
			displayListOfSounds();
			print( 'Select a number between 1 and '..sndListSize );
		else
			cprrest = tonumber( cprrest );
			if( cprrest > 0 and cprrest <= sndListSize ) then
				slicesound = cprrest;
				print( 'Setting SnD warning sound to: '..slicesound..' - '..string.format( soundListFormat, getWSLNameByIndex(slicesound) ) );
			else
				print( 'The value must be between 1 and '..sndListSize );
			end
		end
    -- warning sound selector
    elseif command == 'wsli' then
		sndListSize = getn( warnSoundList );
		if( cprrest == "" ) then
			displayListOfSounds();
			print( 'Select a number between 1 and '..sndListSize );
		else
			cprrest = tonumber( cprrest );
			if( cprrest > 0 and cprrest <= sndListSize ) then
				print( 'Listening to: '..cprrest..' - '..string.format( soundListFormat, getWSLNameByIndex(cprrest) ) );
				PlaySoundFile( getWSLPathByIndex( cprrest ) );
			else
				print( 'The value must be between 1 and '..sndListSize );
			end
		end
    else
		-- getting a shorter list of commands
		cmdShortList = "";
		cmdListSize = getn( cmdList );
        for i,cmd in ipairs(cmdList) do
			if( i < cmdListSize ) then
				cmdShortList = cmdShortList..string.format( cmdListFormat, cmd.name ).." | ";
			else
				cmdShortList = cmdShortList..string.format( cmdListFormat, cmd.name );
			end
        end
        print("Options: /cypr ( "..cmdShortList.." ) ");
        
        -- printing a detailed command list
        print("Explained commands:");
        for i,cmd in ipairs(cmdList) do
			print( string.format( cmdListFormat, cmd.name )..' - '..cmd.descr );
        end
    end
end

SLASH_CYANIDESLASH1, SLASH_CYANIDESLASH2 = '/cypr', '/cyanide';
SlashCmdList["CYANIDESLASH"] = handler;

local addonName, L = ...;
local function defaultFunc(L, key)
return key;
end
setmetatable(L, {__index=defaultFunc});
L["Wound Poison"] = GetSpellInfo(8679)
L["Deadly Poison"] = GetSpellInfo(2823)
L["Crippling Poison"] = GetSpellInfo(3408)
L["Leeching Poison"] = GetSpellInfo(108211)
L["Instant Poison"] = GetSpellInfo(157584)
L["Agonizing Poison"] = GetSpellInfo(200802)
L["Slice and Dice"] = GetSpellInfo(5171);

local function CPR_OnEvent(self, event, ...)
	if (event == "PLAYER_SPECIALIZATION_CHANGED" or event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_TALENT_UPDATE") then
		local spec = GetSpecializationInfo(GetSpecialization() or 1); -- 259 ass, 260 outlaw, 261 sub
		local slice = GetSpellInfo("Slice and Dice");
		canpoison = spec == 259;
		cansnd = spec == 260 and slice ~= nil; -- assassination has passive snd always active, sub doesnt have it, outlaw now a talent
		if debugmode then print("flags",spec,slice,canpoison,cansnd); end
	elseif (event == "PLAYER_ENTERING_WORLD") then
		if debugmode then print("enter world"); end
		StartTimer();
		return;
	elseif (event == "PLAYER_REGEN_DISABLED") then
		if debugmode then print("enter combat"); end
		if slicewarn > 0 and cansnd then
			C_Timer.After(slicewarn,CheckSlice);
		end
	elseif (event == "ZONE_CHANGED_NEW_AREA") then
		if debugmode then print("change zone"); end
	end
	if (canpoison) then
		CheckPoisons();
	end
end

function CheckSlice()
	if (cprstatus ~= "off") then
 		local expslice = select(7,UnitBuff("player", L["Slice and Dice"]));
		local timeNow = GetTime();
		if not expslice then
			WarningHandle("No Slice and Dice active!",timeNow,true,slicesound);
			if (InCombatLockdown()) then
				C_Timer.After(slicewarn,CheckSlice);
			end
		elseif (expslice > timeNow) then
			C_Timer.After(expslice-timeNow,CheckSlice);
		end
	end
end

-- not local, called back by the timer function
function CheckPoisons()
	if (cprstatus ~= "off" and not IsResting() and not UnitOnTaxi("player") and OffhandHasWeapon() and (UnitLevel("player")) >= 10) then	
		local timeNow = GetTime();
		local expmath = timeNow + (cprexp * 60);
		local utilityP = false;
		local damageP = false;
 		local expireCP = select(7,UnitBuff("player", L["Crippling Poison"]));
		local expireDP = select(7,UnitBuff("player", L["Deadly Poison"]));
		local expireWP = select(7,UnitBuff("player", L["Wound Poison"]));
		local expireLP = select(7,UnitBuff("player", L["Leeching Poison"]));
		local expireIP = select(7,UnitBuff("player", L["Instant Poison"]));
		local expireAP = select(7,UnitBuff("player", L["Agonizing Poison"]));
		if debugmode then 
			print("conditions passed, checking poison timers now...");
			print("CP "..(expireCP or "missing"));
			print("DP "..(expireDP or "missing"));
			print("WP "..(expireWP or "missing"));
			print("LP "..(expireLP or "missing"));
			print("IP "..(expireIP or "missing"));
			print("AP "..(expireAP or "missing"));
			print("values below "..expmath.." should trigger a warning");
		end
-- util
		if expireCP then
			utilityP = true;
	        if (expireCP < expmath and utilityWarn == "on" ) then
				WarningHandle(L["Crippling Poison"] .. " Running Out   ",timeNow,false);
	        end
		end
		
		if expireLP then
			utilityP = true;
	        if (expireLP < expmath and utilityWarn == "on") then
				WarningHandle(L["Leeching Poison"] .. " Running Out   ",timeNow,false);
	        end
		end
-- damage	
		if expireDP then
			damageP = true;
	        if (expireDP < expmath) then
				WarningHandle(L["Deadly Poison"] .. " Running Out   ",timeNow,false);
	        end
		end
	
		if expireWP then
			damageP = true;
	        if (expireWP < expmath) then
				WarningHandle(L["Wound Poison"] .. " Running Out   ",timeNow,false);
	        end
		end
		
		if expireIP then
			damageP = true;
			if (expireIP < expmath) then
				WarningHandle(L["Instant Poison"] .. " Running Out   ",timeNow,false);
			end
		end

		if expireAP then
			damageP = true;
			if (expireAP < expmath) then
				WarningHandle(L["Agonizing Poison"] .. " Running Out   ",timeNow,false);
			end
		end
-- missing	
		if ( utilityP == false and (UnitLevel("player")) >= 20 and utilityWarn == "on" ) then
			WarningHandle("No utility poison applied!   ",timeNow,true);
		end
	
		if ( damageP == false ) then
			WarningHandle("No damage poison applied!   ",timeNow,true);
		end
	elseif (debugmode) then
	    print ("skipping checks because of failed conditions, breakdown follows...");
	    print ("CPR status = " ..valueDisplay(cprstatus));
	    print ("not resting = " ..valueDisplay(not IsResting() and "true" or "false"));
	    print ("not on a taxi = " ..valueDisplay(not UnitOnTaxi("player") and "true" or "false"));
	    print ("offhand equipped = " ..valueDisplay(OffhandHasWeapon() and "true" or "false"));
	    print ("at least level 10 = " ..valueDisplay(UnitLevel("player") >= 10 and "true" or "false"));
	end
end

function WarningHandle(message, timeNow, ismissing, soundindex)
	SoundHandle(false, timeNow, soundindex or warnSound);
	if (ismissing) then
		UIErrorsFrame:AddMessage(message, 1.0, 0.0, 0.0, 5.0);
	else
		UIErrorsFrame:AddMessage(message, 1.0, 1.0, 0.5, 5.0);
	end
	DEFAULT_CHAT_FRAME:AddMessage(message);
end

-- playing sound according to the settings
function SoundHandle( isTestCmd, timeNow, soundindex )
	if (isTestCmd or warnSoundState == "on") then
		-- prevent overlapping sound for both poisons
		if (timeNow > lastSound+5) then
			lastSound = timeNow;
			PlaySoundFile( getWSLPathByIndex(soundindex) );
		end
	end
end

-- displaying value according to format
function valueDisplay( val )
	return string.format( valueDisplayFormat, val );
end

-- display list of sounds available
function displayListOfSounds()
	print('Displaying a list of warning sounds:');
	for i,snd in ipairs(warnSoundList) do
		print( i..' - '..string.format( soundListFormat, snd.name ) )
	end
	print('----------------------------------------');
end

-- getting warning sound name according to an index from warnSoundList table
function getWSLNameByIndex( index ) 
	return warnSoundList[index].name;
end

-- getting warning sound path according to an index from warnSoundList table
function getWSLPathByIndex( index ) 
	return warnSoundList[index].path;
end

-- returns a combination of hours/minutes/seconds from a given number of miliseconds
function GetDuration( time )
	time = math.floor( time / 1000 );
	
	-- treating something that is returned as 0ms from GetBattlefieldEstimatedWaitTime
	-- when you join a BG for which there are no instances existing
	-- not 100% correct
	if( time == 0 ) then
		return "< 1 minute"
	end
	
	maxMinutes = 60
	maxHours = 3600
	seconds = time
	minutes = 0
	hours = 0
	
	if( time > maxMinutes - 1 and time < maxHours ) then
		minutes = math.floor( time / maxMinutes )
		seconds = time % maxMinutes
	elseif( time > maxHours - 1 ) then
		hours = math.floor( time / maxHours )
		remainder = time % maxHours
		minutes = math.floor( remainder / maxMinutes )
		seconds = remainder % maxMinutes
	end
	
	-- we return time in hours, minutes and seconds
	if( hours > 0 ) then
		return GetPlural( hours, "hour" ).." , "..GetPlural(  minutes, "minute" ).." , "..GetPlural(  seconds, "second" );
		
	elseif( minutes > 0 ) then -- we return time in minutes and seconds
		return GetPlural(  minutes, "minute" ).." , "..GetPlural(  seconds, "second" );
	else -- we return time in seconds
		return GetPlural(  seconds, "second" );
	end
		
end

-- returns the plural of a measuring unit according to the value
function GetPlural( value, unit )
	result = value.." "
	if( value > 1 or value == 0 ) then
		return result..unit..'s'
	end
	
	return result..unit
end

frame:SetScript("OnEvent", CPR_OnEvent);
